#!/usr/bin/python
## Copyright (C) 2003 Red Hat, Inc.
## Copyright (C) 2003 Dan Walsh <dwalsh@redhat.com>

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
import sys
import getopt
import os
import signal
import gettext
import pxeos
from netboot_util import *
from netboot_gtk import *
from pxeosdialog import *

import diskless
import gtk
import gnome
import gnome.ui
import socket
import gobject
import gtk.glade

import pxeboot
import pxeos
import firsttime

#Columns for "treeview"
COLUMN_PIXBUF=0
COLUMN_HOSTNAME=1
COLUMN_OS=2
COLUMN_IP=3
COLUMN_DATA=4

quiting=0

# I18N
gettext.bindtextdomain(PROGNAME, "/usr/share/locale")
gettext.textdomain(PROGNAME)

try:
    gettext.install(PROGNAME, "/usr/share/locale", 1)
except IOError:
    import __builtin__
    __builtin__.__dict__['_'] = unicode    

gnome.program_init (PROGNAME,VERSION)
FORMALNAME=_("Red Hat Config Network Booting")

class pxeDialog:
    def __init__(self,xml,parent):
        self.parent=parent
        self.pxeDialog=xml.get_widget("pxebootDialog")
        self.ipEntry=xml.get_widget("ipEntry")
        self.snapshotEntry=xml.get_widget("snapshotEntry")
        self.kickstartEntry=xml.get_widget("kickstartEntry")
        self.ethernetOptionmenu=xml.get_widget("ethernetOptionmenu")
        self.fileSelection=xml.get_widget("Fileselection")
#        self.ramdiskSpinbutton=xml.get_widget("ramdiskSpinbutton")
#        self.ramdiskSpinbutton.set_value(10000)
        self.kickstartLabel=xml.get_widget("kickstartLabel")
        self.osOptionmenu=xml.get_widget("pxeOSOptionmenu")
        self.osOptionmenu.connect("changed",self.on_protocol_changed)
        self.serialConsoleCheckbutton=xml.get_widget("serialConsoleCheckbutton")
        self.serialConsoleCheckbutton.connect("toggled",self.on_serialConsole_toggled)
        self.serialUseSystemConsoleCheckbutton = xml.get_widget("systemConsoleCheckbutton")
        self.serialConsoleSpeed = xml.get_widget("serialConsoleSpeed")
        self.serialConsoleParity = xml.get_widget("serialConsoleParity")
        self.serialConsoleDataBits = xml.get_widget("serialConsoleDataBits")
        self.serialConsoleSpeedLabel = xml.get_widget("serialConsoleSpeedLabel")
        self.enableNISDOMAINCheckbutton = xml.get_widget("enableNISDOMAINCheckbutton")
        self.nisDomainEntry = xml.get_widget("nisDomainEntry")
        self.enableNISDOMAINCheckbutton.connect("toggled", self.on_enableNISDOMAIN_toggled)
        self.extraBootOptions = xml.get_widget("extraBootOptions")
        self.extraBootOptionsLabel = xml.get_widget("extraBootOptionsLabel")
        self.extraBootOptions.set_model( gtk.ListStore(gobject.TYPE_STRING) )
        self.extraBootOptions.set_text_column(0)
        self.extraBootOptionsEntry = self.extraBootOptions.child
        self.extraBootOptionsEntry.connect("changed", self.on_bootOption_changed)
        self.generateSnapshotCheckbutton = xml.get_widget("generateSnapshot")
        self.generateSnapshotCheckbutton.connect("toggled",self.on_generateSnapshot_changed)
        self.disableSnapshotGeneration = False
        self.bootOptionSelected = -1
        self.remoteLoggingEnabled = True
        self.remoteLoggingHost = None
        self.remoteLoggingEnabledCheckbutton = xml.get_widget("remoteLoggingEnabled")
        self.remoteLoggingEnabledCheckbutton.connect("toggled",self.on_remoteLoggingEnabled_changed)
        self.remoteLoggingHostEntry = xml.get_widget("remoteLoggingHost")
        
    def __clear(self):
        self.ipEntry.set_text("")
        self.ipEntry.set_sensitive(True)
        self.kickstartEntry.set_text("")
        self.snapshotEntry.set_text("")
        
    def __update(self):
        self.pxe.setKickstart(self.kickstartEntry.get_text())
        snapshot = self.snapshotEntry.get_text()
        if (snapshot != None) and (len(snapshot)):
            self.pxe.setSnapshot(snapshot)
        self.pxe.setEthernet("eth%d"%self.ethernetOptionmenu.get_history())
        self.pxe.setSerial(self.serialConsoleCheckbutton.get_active())        
        if self.serialConsoleCheckbutton.get_active():            
            self.pxe.setSerialUseSystemConsoleAlso( self.serialUseSystemConsoleCheckbutton.get_active())
            o = self.serialConsoleSpeed.get_history()
            if (o > -1) and (o != 4): # non-default speed selected
                self.pxe.setSerialSpeed( o )
            o = self.serialConsoleParity.get_history()
            if o > 0:  # non-default parity selected:
                self.pxe.setSerialParity( o )
            o = self.serialConsoleDataBits.get_history()
            if o > 0:
                self.pxe.setSerialDataBits( o )
#        self.pxe.setRamdisk(int(self.ramdiskSpinbutton.get_value()))        
        os=self.osList[self.osOptionmenu.get_history()]
        if hasattr(self.pxe,'ramdisk') and self.pxe.ramdisk != None:
            if (os != None) and (os.getName() != self.pxe.os):
                self.pxe.ramdisk = None
        self.pxe.setOS(os.getName())
        bo = {}
        ek = self.pxe.dict.keys()
        bom = self.extraBootOptions.get_model()
        bon = bom.iter_n_children(None)
        if bon > 1: # 0th child always empty
            for i in range(1,bon):
                iter = bom.get_iter((i,))
                o = bom.get_value(iter,0)
                e = o.find('=')
                if e > -1:
                    bo[ o[:e] ] = o[e+1:]
                else:
                    bo[o] = None
        ok = bo.keys()
        for k in ok:
            if len(k):
                self.pxe.dict[k] = bo[k]
        for k in ek:
            if not k in ok:
                del self.pxe.dict[k]
        bom.clear()
        if self.enableNISDOMAINCheckbutton.get_active():
            nisDomain = self.nisDomainEntry.get_text()
            if len(nisDomain) and (nisDomain.find(' ') == -1):
                self.pxe.setNISdomain(nisDomain)
            else:
                self.pxe.setNISdomain('')
        else:
            self.pxe.setNISdomain(None)
        self.pxe.disableSnapshotGeneration = self.disableSnapshotGeneration
        if self.remoteLoggingEnabled:
            remoteLogHost = self.remoteLoggingHostEntry.get_text()
            if (remoteLogHost == None) or (len(remoteLogHost)==0):
                remoteLogHost=''
            self.pxe.remoteLogHost=remoteLogHost
        else:
            self.pxe.remoteLogHost=None
    
    def on_protocol_changed(self,item):
        os=self.osList[item.get_history()]
        self.kickstartEntry.set_sensitive(not os.isDiskless())
        self.snapshotEntry.set_sensitive(os.isDiskless())
        self.ethernetOptionmenu.set_sensitive(os.isDiskless())
            
    def showNew(self):
        self.__clear()
        self.pxeDialog.set_title(_("New"))
        self.serialConsoleCheckbutton.set_active(False)
        self.on_serialConsole_toggled(self.serialConsoleCheckbutton)        
        while 1:
            status = self.run(False)
            if status == gtk.RESPONSE_OK:
                try:
                    ip=self.ipEntry.get_text()
                    try: 
                        self.pxe = pxeboot.pxeboot(ip)
                    except:
                        errorMessage("Invalid Hostname/IPAddress")
                        continue
                    self.__update()
                    cursor=Cursor(self.pxeDialog.window)
                    cursor.wait()
                    self.pxe.save()
                    cursor.ready()
                    self.pxeDialog.hide()
                    return status
                except:
                    raise
            else:
                self.pxeDialog.hide()
                return status
                
    def showEdit(self,pxe):
        self.pxe=pxe
        self.__clear()
        self.pxeDialog.set_title(_("Edit: %s") % self.pxe.getHostname())
        self.ipEntry.set_text(self.pxe.getHostname())
        self.ipEntry.set_sensitive(False)
        self.kickstartEntry.set_text(self.pxe.getKickstart())
        snapshot = self.pxe.getSnapshot()
        if snapshot != None:
            self.snapshotEntry.set_text(snapshot)
        self.ethernetOptionmenu.set_history(int(self.pxe.getEthernet()[-1]))
        self.serialConsoleCheckbutton.set_active(self.pxe.isSerial())
        self.on_serialConsole_toggled(self.serialConsoleCheckbutton)
        # self.ramdiskSpinbutton.set_value(self.pxe.getRamdisk())
        self.extraBootOptions.append_text("") # for "NEW" options
        nisdomain = self.pxe.getNISdomain()
        if nisdomain != None:
            self.enableNISDOMAINCheckbutton.set_active(True)
            self.nisDomainEntry.set_sensitive(True)
            if nisdomain != '(none)':
                self.nisDomainEntry.set_text(nisdomain)
        if len( self.pxe.dict.keys() ):
            keys=self.pxe.dict.keys()
            for k in keys:
                if self.pxe.dict[k] != None:
                    self.extraBootOptions.append_text("%s=%s" % (k,self.pxe.dict[k]))
                else:
                    self.extraBootOptions.append_text(k)
        pxeos = self.pxe.getpxeos()
        if pxeos != None:
            loc=pxeos.getLocation()
            if (loc != None) and len(loc):
                (e,o)=getstatusoutput("/bin/sed -n '/^\*\.\*[\ \	]*\@[^\ \	]*/{s/^.*\@\([^\ \	]*\)/\\1/;p}' < " + loc + '/root/etc/syslog.conf')
                if (e == 0) and (len(o) > 0):
                    self.remoteLoggingHostEntry.set_text(o)
                    self.remoteLoggingEnabledCheckbutton.set_active(True)                    
        while 1:
            status = self.run()
            if status == gtk.RESPONSE_OK:
                try:
                    self.__update()
                    cursor=Cursor(self.pxeDialog.window)
                    cursor.wait()
                    self.pxe.save()
                    cursor.ready()
                    self.pxeDialog.hide()
                    return
                except Exception,e:
                    print _("Failed to save PXE boot configuration - error:\n"),e
                    raise
            else:
                bom = self.extraBootOptions.get_model()
                bom.clear()
                self.pxeDialog.hide()
                return
    
    def run(self,modify=True):
        item=0
        ctr=0
        menu=gtk.Menu()
        pxelist=pxeos.pxeOSList()
        self.osList=pxelist.getOSList()
        for OS in self.osList:
            if modify:
                if self.pxe.getOS() ==OS.getName():
                    item=ctr
                else:
                    ctr=ctr+1
            mitem=gtk.MenuItem(OS.getName())
            mitem.show()
            menu.append(mitem)
        if not modify:
            self.extraBootOptions.append_text("") # for "NEW" options
        self.osOptionmenu.set_menu(menu)
        self.osOptionmenu.set_history(item)
        return self.pxeDialog.run()
    
    def getPXE(self):
        return self.pxe
    
    def hide(self,*args):
        self.pxeDialog.hide()
    
    def on_serialConsole_toggled(self, b):
        if self.serialConsoleCheckbutton.get_active() :        
            self.serialUseSystemConsoleCheckbutton.set_sensitive(True)
            if hasattr(self,'pxe'):
                self.serialUseSystemConsoleCheckbutton.set_active(self.pxe.getSerialUseSystemConsoleAlso())
            else:
                self.serialUseSystemConsoleCheckbutton.set_active(True)
            self.serialConsoleSpeed.set_sensitive(True)
            self.serialConsoleSpeedLabel.set_sensitive(True)
            if hasattr(self,'pxe'):
                self.serialConsoleSpeed.set_history(self.pxe.getSerialSpeed(int))
            else:
                self.serialConsoleSpeed.set_history(4)
            self.serialConsoleParity.set_sensitive(True)
            if hasattr(self,'pxe'):            
                self.serialConsoleParity.set_history(self.pxe.getSerialParity(int))
            else:
                self.serialConsoleParity.set_history(0)
            self.serialConsoleDataBits.set_sensitive(True)
            if hasattr(self,'pxe'):
                self.serialConsoleDataBits.set_history(self.pxe.getSerialDataBits(int))
            else:
                self.serialConsoleDataBits.set_history(0)
        else:
            self.serialUseSystemConsoleCheckbutton.set_sensitive(False)            
            self.serialConsoleSpeed.set_sensitive(False)
            self.serialConsoleParity.set_sensitive(False)
            self.serialConsoleSpeedLabel.set_sensitive(False)
            self.serialConsoleDataBits.set_sensitive(False)

    def on_bootOption_changed(self, e):
        bom = self.extraBootOptions.get_model()
        bot = self.extraBootOptionsEntry.get_text()
        boi = self.bootOptionSelected
        i = self.extraBootOptions.get_active()
        if i > -1:
            self.bootOptionSelected = i
        if (self.bootOptionSelected <= 0) and (len(bot)>0):
            # "NEW" option
            iter = bom.append( [ bot ] )
            self.bootOptionSelected = bom.get_path(iter)[0]
        elif self.bootOptionSelected > 0:
            iter = bom.get_iter((self.bootOptionSelected,))
        else:
            return 0
        if (len(bot) == 0) and (boi == self.bootOptionSelected):
            # "DELETE" option:
            bom.remove(iter)
            self.bootOptionSelected = -1
        else:
            # "MODIFY" option:
            bom.set_value(iter,0,bot)
        return 0
    
    def on_enableNISDOMAIN_toggled(self, b):
        if self.enableNISDOMAINCheckbutton.get_active():
            self.nisDomainEntry.set_sensitive(True)
        else:
            self.nisDomainEntry.set_text(_("set $NISDOMAIN"))
            self.nisDomainEntry.set_sensitive(False)

    def on_generateSnapshot_changed(self,b):
        if self.generateSnapshotCheckbutton.get_active():
            self.disableSnapshotGeneration = False;
        else:
            self.disableSnapshotGeneration = True;

    def on_remoteLoggingEnabled_changed(self,b):
        if self.remoteLoggingEnabledCheckbutton.get_active():
            self.remoteLoggingEnabled = True
        else:
            self.remoteLoggingEnabled = False

class Gui:
    """This class handles everything gui for the system-config-services application"""
    def __init__(self):

        signal.signal (signal.SIGINT, signal.SIG_DFL)

        # make sure threading is disabled
        try:
            from gtk import _disable_gdk_threading
            _disable_gdk_threading()
        except ImportError:
            pass
        self.dirty=0

        gladepath = "system-config-netboot.glade"
        if not os.path.exists(gladepath):
            gladepath = "%s/%s" % (INSTALLDIR,gladepath)
        gtk.glade.bindtextdomain(PROGNAME)
        self.xml = gtk.glade.XML(gladepath, domain=PROGNAME)

        # map the event signals to specific methods
        self.xml.signal_autoconnect(
            { "on_mainwin_delete_event" : self.quit,
              "on_mainwin_hide" : self.quit,
              "on_quit_activate" : self.quit,
              "on_about_activate" : self.on_about_activate,
              "on_contents_activate" : self.on_contents_activate,
              "on_closeButton_activate" : self.quit,
              "on_newButton_clicked" :    self.on_newButton_clicked,
              "on_editButton_clicked" :   self.on_editButton_clicked,
              "on_deleteButton_clicked" : self.on_deleteButton_clicked,
              "on_refreshDiskless_activate" : self.on_refreshDiskless_activate,
              "on_firsttime_druid_activate" : self.on_firsttime_druid_activate,
              "on_diskless_activate" : self.on_diskless_activate,
              "on_operating_systems_activate" : self.on_operating_systems_activate,
              } )

        self.disklessButton = self.xml.get_widget("disklessButton")
        self.bootTree = self.xml.get_widget("bootfileTreeview")
        self.pxeDialog = pxeDialog(self.xml,self)
        self.winMain = self.xml.get_widget("netbootApp")
        self.netbootFirsttime = firsttime.FirstTime(self.xml,self)
        self.firsttime = self.xml.get_widget("firstTimeDialog")
        self.winMain.connect("delete-event", self.quit)
        self.winMain.connect("hide", self.quit)
        self.pxelistdialog=pxelistdialog(self.xml)
        self.setupList()

    def show(self):
        self.winMain.show()

    def on_contents_activate(self,args):
        page = "file:///usr/share/doc/"+ PROGNAME + "-" + VERSION + "/index.html"
        gnome.url_show(page)

    def on_about_activate (self,*args):
        dialog = gnome.ui.About(PROGNAME,
                                VERSION,
                                '(C) 2003 - 2006 Red Hat, Inc.',
                                FORMALNAME,
                                [ 'Daniel Walsh <dwalsh@redhat.com>', 'Jason Vas Dias<jvdias@redhat.com>'],[ 'Tammy Fox <tfox@redhat.com>'],_("translator_credits"))
        dialog.set_title (FORMALNAME)
        dialog.show()
        
    def getImage(self,pxe):
        if pxe.isDiskless():
            return gtk.gdk.pixbuf_new_from_file("%s/diskless.gif" % INSTALLDIR)
        else:
            return gtk.gdk.pixbuf_new_from_file("%s/install.gif" % INSTALLDIR)
        
    def on_newButton_clicked(self,args):
        if self.pxeDialog.showNew() == gtk.RESPONSE_OK:
            pxe=self.pxeDialog.getPXE()
            model=self.bootTree.get_model ()
            iter = model.append ()
            model.set (iter,
                       COLUMN_PIXBUF,  self.getImage(pxe),
                       COLUMN_HOSTNAME,  pxe.getHostname(),
                       COLUMN_OS,pxe.getOS(),
                       COLUMN_IP,pxe.getIP(),
                       COLUMN_DATA, pxe)
            self.bootTree.get_selection().select_iter (iter)
            selection=self.bootTree.get_selection()
            self.on_tree_view_selection_changed (selection)
        
    def on_deleteButton_clicked(self,args):
        """Callback for the deleteButton."""
        result = self.bootTree.get_selection().get_selected ()
        if result == None:
            return 
        else:
            (model, iter) = result
            try:
                pxe = model.get_value (iter, COLUMN_DATA)
            except TypeError:
                return
            message=_("Are you sure you want to delete the '%s' network boot file?") % pxe.getHostname()
            dlg = gtk.MessageDialog(self.winMain, 0, gtk.MESSAGE_INFO,
                                    gtk.BUTTONS_YES_NO,
                                    message)
            dlg.show_all()
            rc = dlg.run()
            dlg.destroy()
            if rc==gtk.RESPONSE_YES:
                pxe.delete()
                model.remove(iter)

    def on_operating_systems_activate(self,button):
        self.pxelistdialog.run()
        
    def on_diskless_activate(self,button):
        self.pxelistdialog.run(TRUE)
        
    def on_firsttime_druid_activate(self,button):
        rc=self.firsttime.run()
        self.firsttime.hide()
        OS=None
        if rc==1:
            OS=self.pxelistdialog.runNetboot()
        if rc==2:
            OS=self.pxelistdialog.runDiskless()
        if OS!=None:
            pxeos.pxeOSList().append(OS)

        return rc
        
    def on_refreshDiskless_activate(self,button=None):
        """Callback for the disklessButton."""
        result = self.bootTree.get_selection().get_selected ()
        if result == None:
            return 
        else:
            (model, iter) = result
            try:
                pxe = model.get_value (iter, COLUMN_DATA)
                message=_("Are you sure you want to refresh the '%s' diskless environment?\nThis could take several minutes to run.") % pxe.getHostname()
                dlg = gtk.MessageDialog(self.winMain, 0, gtk.MESSAGE_INFO,
                                        gtk.BUTTONS_YES_NO,
                                        message)
                dlg.show_all()
                rc = dlg.run()
                dlg.destroy()
                if rc==gtk.RESPONSE_YES:
                    d=diskless.Diskless(pxe.getHostname())
                    d.setup()

            except TypeError:
                pass
                
        
    def on_editButton_clicked(self,button=None):
        selection=self.bootTree.get_selection()
        result = selection.get_selected ()
        if result == None:
            return 
        else:
            (model, iter) = result
            try:
                pxe = model.get_value (iter, COLUMN_DATA)
                self.pxeDialog.showEdit(pxe)
                model.set (iter,
                           COLUMN_PIXBUF,  self.getImage(pxe),
                           COLUMN_HOSTNAME,  pxe.getHostname(),
                           COLUMN_OS,pxe.getOS(),
                           COLUMN_IP,pxe.getIP(),
                           COLUMN_DATA, pxe)
                self.on_tree_view_selection_changed (selection)
            except TypeError:
                pass

    def populateList(self):
        PXEFiles = pxeboot.getPXEFiles()
        model=self.bootTree.get_model ()
        for i in PXEFiles.keys():
            try:
                pxe=pxeboot.pxeboot(PXEFiles[i])
                pxe.load()
            except IOError,message:
                rc=yesNoQuestion(("\nError:Invalid PXE Boot File %s%s\n%s\nDo you wish to delete PXE Boot File?" % (PXECFGDIR,i,message)))
                if rc==gtk.RESPONSE_YES:
                    os.remove("%s%s" % (PXECFGDIR,i))
                continue
            except ValueError,message:
                rc=yesNoQuestion(_("\nError:Invalid PXE Boot File %s%s\n%s\nDo you wish to delete PXE Boot File?" % (PXECFGDIR,i,message)))
                if rc==gtk.RESPONSE_YES:
                    os.remove("%s%s" % (PXECFGDIR,i))
                continue
            except:
                rc=yesNoQuestion(_("\nError:Invalid PXE Boot File %s%s\nDo you wish to delete PXE Boot File?" % (PXECFGDIR,i)))
                if rc==gtk.RESPONSE_YES:
                    os.remove("%s%s" % (PXECFGDIR,i))
                continue
            
            iter = model.append ()
            model.set (iter,
                       COLUMN_PIXBUF,  self.getImage(pxe),
                       COLUMN_HOSTNAME,  pxe.getHostname(),
                       COLUMN_OS,pxe.getOS(),
                       COLUMN_IP,pxe.getIP(),
                       COLUMN_DATA, pxe)
            selection=self.bootTree.get_selection()
            selection.select_path ((0,))
            self.on_tree_view_selection_changed (selection)

    def on_tree_view_row_activated (self,tree_view, path, column, func):
        apply (func)

    def on_tree_view_selection_changed (self,selection):
        pass

    def compareIP(self, treemodel, iter1, iter2, user_data):
        """Compare IP addresses numerically"""
        val1 = treemodel.get_value(iter1, user_data)
        val2 = treemodel.get_value(iter2, user_data)
        ip1 = [int(x) for x in val1.split(".")]
        ip2 = [int(x) for x in val2.split(".")]
        if ip1 == ip2:
            return 0
        elif ip1 < ip2:
            return -1
        else:
            return 1

    def setupList(self):
        """Populates bootfileTreeview with the bootfile names"""
        self.bootTree = self.xml.get_widget("bootfileTreeview")
        model = gtk.ListStore (gtk.gdk.Pixbuf, gobject.TYPE_STRING,gobject.TYPE_STRING, gobject.TYPE_STRING,gobject.TYPE_PYOBJECT)
        model.set_sort_column_id(1, gtk.SORT_ASCENDING)        
        model.set_sort_func(3, self.compareIP, 3)
        self.bootTree.set_model (model)
        column = gtk.TreeViewColumn (None, gtk.CellRendererPixbuf(), pixbuf=COLUMN_PIXBUF)
        self.bootTree.append_column (column)
        
        column = gtk.TreeViewColumn (_("Hostname"), gtk.CellRendererText(), text=COLUMN_HOSTNAME)
        column.set_sort_column_id(1)
        self.bootTree.append_column (column)

        column = gtk.TreeViewColumn (_("Operating System"), gtk.CellRendererText(), text=COLUMN_OS)
        column.set_sort_column_id(2)
        self.bootTree.append_column (column)

        column = gtk.TreeViewColumn (_("IP Address"), gtk.CellRendererText(), text=COLUMN_IP)
        column.set_sort_column_id(3)
        self.bootTree.append_column (column)
        self.populateList()
            
        self.bootTree.get_selection().connect ("changed",
                                               self.on_tree_view_selection_changed)
        self.bootTree.connect ("row_activated",self.on_tree_view_row_activated,
                               self.on_editButton_clicked)

    def quit(self,arg1=None,arg2=None):
        global quiting
        quiting=1
        gtk.main_quit()    


if __name__ == "__main__":
    try:
        gui=Gui()
        if pxeos.firsttime():
            rc=gui.on_firsttime_druid_activate(None)
            if rc != gtk.RESPONSE_CANCEL:
                gui.show()
            else:
                sys.exit(0)
        else:
            gui.show()
                
        gtk.main()
    except:
        # if you quit during initialization exceptions can be raised
        if not quiting:
            raise
